<?php

/**

 WDB
 MySQL database connection and functions

**/

class WDB
{
	private $dbLink = NULL;
	private $dbResult = NULL;
	private $fException = true;
	
	public function __construct ($host, $user, $password, $name) {
		$this->create ($host, $user, $password, $name);
	}
	
	public function __destruct () {
		try {
			$this->close ();
		}
		catch (Exception $e) {
		}
	}	
	
	public function setException ($on) {
		$this->fException = $on;	
	}
	
	public function create ($host, $user, $password, $name) {	
		$this->dbLink = mysqli_connect ($host, $user, $password);
		if ($this->dbLink) {
			
			if (!mysqli_select_db ($this->dbLink, $name)) {
				if ($this->fException)
					$this->throwError ("Cannot select database.");
				return false;
			}
		}
		else {
			if ($this->fException)
				$this->throwError ("Cannot connect database.");
			return false;
		}
		return true;
	}
	
	public function close () {
		if ($this->dbResult) {
			mysqli_free_result ($this->dbResult);
			$this->dbResult = NULL;
		}
		if ($this->dbLink) {
			mysqli_close ($this->dbLink);
			$this->dbLink = NULL;
		}
	}

	public function query ($sql) {
			
		if ($this->dbResult) {
			mysqli_free_result ($this->dbResult);
			$this->dbResult = NULL;
		}
		
		$dbResult = mysqli_query ($this->dbLink, $sql);	
		if (!$dbResult) {
			if ($this->fException)
				$this->throwError ("Cannot query database.");
			return false;
		}
		

		foreach (array ("select", "show") as $cmd) {
			$pos = stripos (trim ($sql), $cmd);
			if ($pos!==false && $pos==0) {
				$this->dbResult = $dbResult;
				return mysqli_num_rows ($this->dbResult);
			}
		}
		return mysqli_affected_rows ($this->dbLink);
	}
	
	public function queryRow ($table, $extraClause, &$row, $columns=NULL, $fIndex=false) {

		$sql = "select ".self::createColumns ($columns)." from ".$table." ".$extraClause;

		if ($this->query ($sql)) {
			while ($this->fetch ($row, $fIndex)) {
				
			}
			return true;
		}
		return false;
	}
	

	public function queryTable ($table, $idColumn, $extraClause="", $column="", $columns=NULL) {

		$resultList = array ();
		if (is_array ($columns))
			array_push ($columns, $idColumn);
		$sql = "select ".self::createColumns ($columns)." from ".$table." ".$extraClause;

		if ($this->query ($sql)) {
			while ($this->fetch ($row)) {
				if (!empty ($column))
					$resultList[$row[$idColumn]]=$row[$column];
				else
					$resultList[$row[$idColumn]]=$row;
			}
		}
		return $resultList;
	}
	
	public function getId ($sql, $idColumn, $fRowCnt=false) {
		$id = 0;
		if (($rowCnt=$this->query ($sql))) {
			if ($fRowCnt)
				return $rowCnt;
			$row = array ();
			
			while ($this->fetch ($row)) {
				if (isset ($row[$idColumn])) {
					$id = $row[$idColumn];
				}
			}
		}
		return $id;
	}
	
	public function fetch (&$row, $fIndex=false) {
		if ($this->dbResult) {
			if ($row2=($fIndex ? mysqli_fetch_row ($this->dbResult) : mysqli_fetch_assoc ($this->dbResult))) {
				$row = $row2;
				return true;
			}
			mysqli_free_result ($this->dbResult);
			$this->dbResult = NULL;
		}
		return false;
	}
	
	public function insert ($table, $values) {		
		$sqlKeys = $sqlValues = '';
		foreach ($values as $key => $value) {
		
			if (is_array ($value))
				$value="";
				
			if (!empty ($sqlKeys))
				$sqlKeys.=",";
			$sqlKeys.=$key;
			if (!empty ($sqlValues))
				$sqlValues.=",";
			$sqlValues.="'".WDB::escape ($value)."'";
		}
		
		$sql = 'insert into '.$table.' ('.$sqlKeys.') values ('.$sqlValues.');';

		$this->query ($sql);
		return $this->getInsertId ();
	}
	
	public function update ($table, $values, $keyColumn, $keyId) {
		$sqlValues = '';
		foreach ($values as $key => $value) {
			if (!empty ($sqlValues))
				$sqlValues.=",";
			$sqlValues.=($key."='".WDB::escape ($value)."'");
		}		
		$sql = 'update '.$table.' set '.$sqlValues.' where '.$keyColumn.'=\''.WDB::escape ($keyId).'\'';

		return $this->query ($sql);
	}
	
	public function insertFields ($fields, $table, $values) {	
		$sqlKeys = $sqlValues = '';
		foreach ($fields as $key => $tmp) {
			$list = explode (';', $tmp);
			if (count ($list)<2)
				continue;
			$type     = $list[0];
			$mode     = (count ($list)>1) ? $list[1] : "*";
			$value    = (count ($list)>2) ? $list[2] : WFormat::getEmptyValue ($type);

			if ($mode=="&")
				continue;

				
			if (isset ($values[$key]))
				$value = $values[$key];
			else
			if ($mode=="$")
				continue;
				
			if (!empty ($sqlKeys))
				$sqlKeys.=",";
			$sqlKeys.=$key;
			if (!empty ($sqlValues))
				$sqlValues.=",";

				
			$sqlValues.="'".WDB::escape ($value)."'";
		}
		$sql = 'insert into '.$table.' ('.$sqlKeys.') values ('.$sqlValues.');';

		$this->query ($sql);
		return $this->getInsertId ();
	}
	
	public function updateFields ($fields, $table, $values, $keyColumn, $keyId) {	
		$sqlValues = '';
		foreach ($fields as $key => $tmp) {
			$list = explode (';', $tmp);
			if (count ($list)<2)
				continue;
			$type     = $list[0];
			$mode     = (count ($list)>1) ? $list[1] : "*";
			$value    = (count ($list)>2) ? $list[2] : WFormat::getEmptyValue ($type);
			if ($mode=="&")
				continue;

			if (isset ($values[$key]))
				$value = $values[$key];
			else
			if ($mode=="$")
				continue;
				
			if (!empty ($sqlValues))
				$sqlValues.=",";
			$sqlValues.=($key."='".WDB::escape ($value)."'");
		}
		$sql = 'update '.$table.' set '.$sqlValues.' where '.$keyColumn.'=\''.$this->escape ($keyId).'\'';

		return $this->query ($sql);
	}	
	
	public function delete ($table, $keyColumn, $keyId) {
		$sql = 'delete from '.$table.' where '.$keyColumn.'=\''.$this->escape ($keyId).'\'';
		return $this->query ($sql);
	}
		
	public function getInsertId () {
		return mysqli_insert_id ($this->dbLink);
	}

	public function getLastErrorNo () {
		return mysqli_errno ($this->dbLink);
	}
	
	public function getLastErrorText () {
		return mysqli_error ($this->dbLink);
	}

	
	public static function escape ($str) {
		$search = array("\\",  "\x00", "\n",  "\r",  "'",  '"', "\x1a");
		$replace = array("\\\\","\\0","\\n", "\\r", "\'", '\"', "\\Z");
		return str_replace($search, $replace, $str);
		// return WDB::escape  ($str);
	}
	
	public static function getDate ($fTime=true, $time=-1) {
		if ($fTime)
			return date ("Y-m-d H:i:s", ($time==-1 ? time () : $time));
		return date ("Y-m-d", ($time==-1 ? time () : $time));
	}
	
	public static function isDateEmpty ($date) {	
		return $date=="0000-00-00 00:00:00" || $date=="0000-00-00";
	}
	
	public function checkDuplicate ($table, $valueColumn, $value, $idColumn, $id=0, $returnCol="", $andClause="") {
		$extra = "";

		if ($returnCol!="")
			$extra.=",".$returnCol;
		$sql = "select ".$idColumn.",".$valueColumn.$extra." from ".$table." where ".$valueColumn."='".WDB::escape($value)."' ".$andClause;

		$found = false;	
		if ($this->query ($sql)) {
			$dbRow = array ();
			while ($this->fetch ($dbRow)) {
				if ($dbRow[$idColumn]!=$id) {

					if ($returnCol!="")
						$found = $dbRow[$returnCol];
					else
						$found = !empty ($dbRow[$valueColumn]);
				}
			}
		}
		return $found;
	}

	static public function addWhereClause (&$sql, $column, $value, $fLike=0, $otherColumns="") {
		if ($sql=="")
			$sql.=" where";
		else
			$sql.=" and ";
		$sql.="(".$column;
		if ($fLike) {
			switch ($fLike) {
				case "r":
					$sql.=" like '".WDB::escape (str_replace ("*","%",$value))."%'";
					break;
				case "l":
					$sql.=" like '%".WDB::escape (str_replace ("*","%",$value))."'";
					break;
				
				default:
					$sql.=" like '%".WDB::escape (str_replace ("*","%",$value))."%'";
					break;

			}
		}
		else
			$sql.="='".WDB::escape ($value)."'";
		if (!empty ($otherColumns)) {
		
			$list = explode (",", $otherColumns);
			foreach ($list as $otherColumn) {
				$sql.=" or ".$otherColumn;
				if ($fLike)
					$sql.=" like '%".WDB::escape ($value)."%'";
				else
					$sql.="='".WDB::escape ($value)."'";
			}
		}
			
		$sql.=")";		
	}
	
	static private function createColumns (&$columns) {
		$sqlCol = "";
		if (!is_array ($columns)) {
			$sqlCol = "*";		
		}
		else {

			foreach ($columns as $column) {
				if (!empty ($sqlCol))
					$sqlCol.=",";
				$sqlCol.=$column;
			}
		}
		return $sqlCol;
	}
		
	private function throwError ($text) {

		throw new Exception($text." ".mysqli_errno ($this->dbLink)." / ".mysqli_error ($this->dbLink));
	}

};
?>