<?php

/**

 WUIList

**/

abstract class WUIList extends WUIElement
{
  const RESET_NEVER = "never";
  const RESET_EVER = "ever";
	const RESET_NO_IDX = "no_idx";
	const RESET_INIT = "init";
	
	const DEFAULT_ROW_CNT = 20;
	const DEFAULT_NAV_CNT = 10;

	private $sortInfo = array ("", false, "s");
	private $resetMode = self::RESET_INIT;
	
	private $rowStyleFunc = NULL;
	private $wrapListFunc = NULL;
	protected $rowDataFunc = NULL;

	
	/* Overwritten by child */
	public abstract function getDefinition ();
	public abstract function loadDataList ();
	public abstract function swapSortPos ($nav_idx, $nav_idx2);
	public abstract function selectItem ($nav_idx);
	
	
	private function cmpFunc ($value1, $value2)  {
		$v1   = "";
		$v2   = "";
		$subs = explode ("+", $this->sortInfo[0]);
		while (list ($i,$column)=each ($subs)) {
			$column = str_replace (array ("."), array ("_"), $column);
			
			$column = trim ($column, '~');
			if ($i>0 && $v1!="")
				$v1.="&nbsp;";
			if ($i>0 && $v2!="")
				$v2.="&nbsp;";
			$v1.=$value1[$column];
			$v2.=$value2[$column];
		}
		switch ($this->sortInfo[2]) {
			case "d":
			case "s":
			case "s_html":
			case "www":
			case "www2":
			case "t":
			case "t2":
				if ($this->sortInfo[1])
					return strcmp (strtolower($v1), strtolower($v2));
				else
					return strcmp (strtolower($v2), strtolower($v1));
				break;
			default:
				if ($this->sortInfo[1])
					return $v1 > $v2;				
				else
					return $v1 < $v2;
				break;
		};
		return 0;
	}
	
	private function createSortURI ($col) {
		return WSystem::getRequestURI (array ($this->id."_sort_col_id"=>$col, $this->id."_nav_idx"=>1, $this->id."_is_nav"=>1, $this->id."_is_ajax"=>1, 
			"view_prev" => WSystem::getParam ("view", "", true), "view" => WSystem::getParam ("view", "", true)));
	}
	
	private function createNavURI ($nav_idx) {
		return WSystem::getRequestURI (array ($this->id."_nav_idx"=>$nav_idx, $this->id."_is_nav"=>1, $this->id."_is_ajax"=>1, 
			"view_prev" => WSystem::getParam ("view", "", true), "view" => WSystem::getParam ("view", "", true)),true);
	}

	private function createEditURI ($idx, $editParams) {
		$url =  WSystem::getRequestURI (array ($this->id."_nav_idx"=>($idx+1), "view_prev" => WSystem::getParam ("view", "", true))+$editParams, false, 
			array ($this->id."_sort_col_id"=>1, $this->id."_is_nav"=>1, $this->id."_is_ajax"=>1));
		
		return $url;
	}

	
	public function renderElement () {

		$fReset = false;
		switch ($this->resetMode) {
			case self::RESET_NO_IDX:
				if (!$this->getSelectedIdx ())
					$fReset = true;
				break;
			case self::RESET_INIT:
				if (WSystem::getParam ("view")=="init")
					$fReset = true;
				break;
			case self::RESET_EVER:
				$fReset = true;
				break;
			default:
				// From to view
				if (WSystem::getParam ("view_prev")==$this->resetMode) {
					$fReset = true;
				}
				break;
		};
		if (!isset ($_SESSION[$this->id."_dataList"]) || $fReset)
			$this->reset ($this->getSearchData ());
		
		if (!$this->getSize ()) {
			$this->html = "<div id=\"wrap_".$this->id."\"></div>";
			return;
		}
			
		$dataList = &$_SESSION[$this->id."_dataList"];

		$def = $this->getDefinition ();

		if (!isset ($def["row_cnt"]))
			$def["row_cnt"]=self::DEFAULT_ROW_CNT;
	
		if (!isset ($def["nav_cnt"]))
			$def["nav_cnt"]=self::DEFAULT_NAV_CNT;
			
		if (!isset ($def["listview"]))
			$def["listview"] = "dynamic";
			
		if (!isset ($def["width"]))
			$def["width"] = "100%";

		
		$rowCnt= $def["row_cnt"]; 
		$columns = $def["columns"];
		$navCnt = $def["nav_cnt"];
		$fNav = (int) WSystem::getParam ($this->id."_is_nav");
		$nav_idx = (int) WSystem::getParam ($this->id."_nav_idx");
		$nav_idx2 = (int) WSystem::getParam ($this->id."_nav_idx2");
		$cmd = WSystem::getParam ($this->id."_cmd");
		if ($cmd=="select") {
			$this->selectItem ($nav_idx);		
		}
		else
		if ($cmd=="swap_sort_pos") {
			$this->swapSortPos ($nav_idx, $nav_idx2);
			$nav_idx = $nav_idx2;
		}
		if ($nav_idx>0)
			$page  = (int) (($nav_idx-1)  / $rowCnt);
		else
			$page  = 0;

		$nav_idx0	= ($page * $rowCnt)+1;

		// Get right sort column id
		$sortColumnId  = 0;
		$sortColumnId2 = WSystem::getArrayValue ($def, "sort_col_id", 1);
		$sortDirection = WSystem::getArrayValue ($def, "sort_direction", true);

		if (isset ($_SESSION[$this->id."_sort_col_id"]))
			$sortColumnId2 = $_SESSION[$this->id."_sort_col_id"];
			
		if (isset ($_SESSION[$this->id."_sort_direction"]))
			$sortDirection = $_SESSION[$this->id."_sort_direction"];
			
		if (WSystem::isParam ($this->id."_sort_col_id")) {
			$sortColumnId = (int) WSystem::getParam ($this->id."_sort_col_id");
			if ($sortColumnId2==$sortColumnId) {
				// Change sort direction
				$sortDirection ^= true;
				$_SESSION[$this->id."_sort_direction"]=$sortDirection;
			}
			$_SESSION[$this->id."_sort_col_id"] = $sortColumnId2 = $sortColumnId;
		}
		
		$htmlTable  = "";
		$htmlNavBar = "";
		
		// Header	
		$i=1;
		$htmlTable.="<tr class=\"header\">";
		if (isset ($def["edit_params"]))
			$htmlTable.="<td width=\"16\">&nbsp;</td>";


		reset ($columns);
		while (list ($column, $data) = each ($columns)) {
			if ($data[2]==0)
				continue;
			if ($i==$sortColumnId) {
				$this->sortInfo = array ($column, $sortDirection, $data[0]);
				usort ($dataList, array($this, "cmpFunc"));
			}	
			$extra=""; $extra2="";
			if (isset($data[3]))
				$extra = "align=\"".$data[3]."\" ";
			if ($i==$sortColumnId2) {
				if ($sortDirection)
					$extra2 = "<img width=\"16\" height=\"16\" src=\"images/arrow_up.gif\" alt=\"\" />";
				else
					$extra2 = "<img width=\"16\" height=\"16\" src=\"images/arrow_down.gif\" alt=\"\" />";
			}
			if (strpos ($def["listview"], "dynamic")===false)
				$htmlTable.="<td ".$extra." width=\"".$data[2]."\"><!-->".$data[1]."<!--></td>";
			else
				$htmlTable.="<td onclick=\"WUI.ajaxRequest ('".$this->createSortURI($i)."','wrap_".$this->id."');return false;\" onmouseover=\"this.className='highlight2'\" onmouseout=\"this.className='header'\" ".$extra." width=\"".$data[2]."\"><!-->".$data[1]."<!-->&nbsp;".$extra2."</td>";

			$i++;
		}
		if (strpos ($def["listview"], "sort")!==false) {
			$htmlTable.="<td><img width=\"16\" height=\"1\" src=\"images/spacer.gif\" alt=\"\" /></td><td><img width=\"16\" height=\"1\" src=\"images/spacer.gif\" alt=\"\" /></td>";
		}
		if (strpos ($def["listview"], "select")!==false) {
			$htmlTable.="<td><img width=\"16\" height=\"1\" src=\"images/spacer.gif\" alt=\"\" /></td>";		
		}
		
		
		$htmlTable.="</tr>";

	
		reset ($dataList);
		$idx=$nav_idx0-1;
		
		$y = 0;
		while ($y<$rowCnt) {
		
			if ($idx==count ($dataList))
				break;
			
			$rowData    = $dataList[$idx];
			$fDeleted   = isset ($rowData["is_deleted"]) && $rowData["is_deleted"]==1;
			$fHighlight = false;
			if (isset ($_SESSION[$this->id."_highlightKeyValue"])) {
				$checkHighlight = $_SESSION[$this->id."_highlightKeyValue"];
				if ($rowData[$checkHighlight[0]]==$checkHighlight[1])
					$fHighlight = true;
			}

			
			$extra = "";		
			if (!$fNav && ($idx+1)==$nav_idx)
				$extra = "class=\"highlight\"";
				
			if ($fHighlight)
				$extra = "class=\"highlight3\"";
		
			if ($this->rowStyleFunc) {

				$style = call_user_func ($this->rowStyleFunc, $rowData);
				if ($style!="")
					$extra=' '.$style;
			}

			if (isset ($rowData["is_selected"]) && $rowData["is_selected"]) {
				$extra = " style=\"background-color:yellow;\" ";
			}
				
			if ($fDeleted) {
				$htmlTable.="<tr class=\"deleted\"><td><img src=\"images/deleted.gif\" border=\"0\" alt=\"Gelöscht\" /></td>";
			}
			else {
				// if (strpos ($def["listview"], "dynamic")!==false)
					$extra.=" onmouseover=\"this.className='highlight'\" onmouseout=\"this.className='normal'\"";
				$htmlTable.="<tr ".$extra.">";			
				if (isset ($def["edit_params"]))
					$htmlTable.="<td><a href=\"\" onclick=\"WUI.".WUI::$ajaxMethod." ('".$this->createEditURI ($idx, $def["edit_params"]+array ("page_is_ajax"=>1))."','".WUI::$subPageId."');return false;\" title=\"".WLanguage::EDIT."\"><img src=\"images/edit.png\" width=\"16\" height=\"16\" border=\"0\" alt=\"".WLanguage::EDIT."\" /></a></td>";
			}

			reset ($columns);
			while (list ($column, $colData) = each ($columns)) {
		
				if ($colData[2]==0)
					continue;
			
				$column = trim ($column, '~');

				if (strpos ($column,".")!==false) {
					$column = str_replace (".", "_", $column);
				}
				$value = "";				
				if (strpos ($column, "|")!==false) {
					$subs  = explode ("|",$column);
					while (list ($i,$column)=each ($subs)) {
						if (!empty ($rowData[$column])) {
							$value=$rowData[$column];
							break;
						}
					}
				}
				else {				

					$subs  = explode ("+",$column);
					while (list ($i,$column)=each ($subs)) {
						if ($i>0 && !empty ($value))
							$value.=($colData[0]=="list" ? "," : " ");	
						$value.=$rowData[$column];
					}
				}
				
				$extra="";
				if (isset($colData[3]))
					$extra = "align=\"".$colData[3]."\"";
	
				// Format data types
				switch ($colData[0]) {
					case "s": if (strlen ($value)>255) $value=mb_substr  ($value,0,255,'UTF-8')."..."; $value=htmlspecialchars ($value); break;
					case "s_html": if (strlen ($value)>255) $value=mb_substr  (strip_tags($value),0,255,'UTF-8')."..."; $value=strip_tags ($value); break;
					case "d": $value = WFormat::formatDate ($value); break;
					case "t": $value = WFormat::formatDateTime ($value); break;
					case "t2": $value = WFormat::formatDateTime ($value,true); break;
					case "www": $value = WFormat::formatWebsiteLink ($value); break;
					case "www2": 
						if ($value!="") { 
							$contentTag = WLanguage::CLICK;
							if (isset ($colData[4]) && !empty ($value)) {
								$previewURL = "../wlib/php/utils/uploader/uploader.php?s=".session_name()."&f=".$colData[4]."/".$value."&v=preview";
								$ext = strtolower (WUtils::getFileExtension ($value));
								if ($ext=="jpg" || $ext=="png" || $ext=="gif")
									$contentTag = "<img onmouseout=\"this.style.height='33px';this.style.width='40px';\" onmouseover=\"this.style.height='200px';this.style.width='240px';\" width=\"40\" height=\"33\"  src=\"".$previewURL."\" />";
								// onmouseout=\"this.style.height='33px';this.style.width='40px';\" onmouseover=\"this.style.height='200px';this.style.width='240px';\"
	
								$value = "../wlib/php/utils/uploader/uploader.php?s=".session_name()."&f=".$colData[4]."/".$value."&v=fullview";
								
							}
							else {
								$value = WFormat::formatWebsiteLink ($value, true);
							}
							$value = "<a onclick=\"return !WUI.createMediaPlayer(this.href);\" ".(strpos ($value,"@")!==false ? "" : "target=\"_blank\"")." href=\"".$value."\">".$contentTag."</a>"; 
						}; 
						break;
					case "c": $value = WFormat::formatCurrency ($value,true,true); $extra.=" nowrap"; break;
					case "c2": $value = WFormat::formatCurrency ($value,true,false); $extra.=" nowrap"; break;
					case "f": $value = WFormat::formatNumber ($value); break;
					case "kb": if ($value) { $value = WFormat::formatKBText ($value); } else { $value=""; }; break;;
					case "img_list": 

						if (isset ($colData[4][$value]) && !empty ($colData[4][$value])) {
							$value='<img src="'.$colData[4][$value].'" alt="" />';
						}
						else {
							$value = "";
						}
						break;
						
					case "list": 
						
						$tokens = explode (",", $value);

						$value = "";
						while (list ($i,$token)=each ($tokens)) {		
							$i*=2;
							if (!empty ($value))
								$value.=" ";				
							if (isset ($colData[$i+4][$token])) {
								if (isset ($colData[$i+5]) && !empty ($colData[$i+5]))
									$value.= $colData[$i+4][$token][$colData[$i+5]];
								else
									$value.= $colData[$i+4][$token];  
							}
							
						}
						break;
					case "year_month":
						if ($value==0) 
							$value="---"; 
						else {
							$value = sprintf ("%d / %02d", (int) ($value / 12), (int) ($value % 12) + 1); 
						}
						break;
					case "image":
						$reloader="";
						if (($idx+1)==$nav_idx && (int) WSystem::getParam("is_refresh_img")==1)
							$reloader = "?".time();
						if ($value!="")
							$value = "<img onmouseover=\"Tip('<img src=$value>');\" src=\"".$value.$reloader."\" width=\"".WSystem::getArrayValue($colData,4,100)."\" alt=\"\" />";
						break;
						
					default:
						$value = WFormat::format ($value, $colData[0]);
						break;
					
				}			
				$htmlTable.="<td ".$extra.">".$value."</td>";
			}
			
			if (strpos ($def["listview"], "sort")!==false) {
				// Manual sorting
				if ($idx==0)
					$htmlTable.="<td style=\"width:16px;\"></td>";
				else
					$htmlTable.="<td><a title=\"".WLanguage::MOVE_UP."\" href=\"\" onclick=\"WUI.ajaxRequest ('".WSystem::getRequestURI(NULL, false, array ($this->id."_nav_idx"=>1))."','wrap_".$this->id."',null,false,'".$this->id."_cmd=swap_sort_pos&".$this->id."_nav_idx=".($idx+1)."&".$this->id."_nav_idx2=".($idx)."');return false;\"><img width=\"16\" height=\"16\" src=\"images/arrow_up.gif\" alt=\"\" /></a></td>";
			
				if ($idx+1>=count ($dataList))
					$htmlTable.="<td style=\"width:16px;\"></td>";
				else
					$htmlTable.="<td><a title=\"".WLanguage::MOVE_DOWN."\" href=\"\" onclick=\"WUI.ajaxRequest ('".WSystem::getRequestURI(NULL, false, array ($this->id."_nav_idx"=>1))."','wrap_".$this->id."',null,false,'".$this->id."_cmd=swap_sort_pos&".$this->id."_nav_idx=".($idx+1)."&".$this->id."_nav_idx2=".($idx+2)."');return false;\"><img width=\"16\" height=\"16\" src=\"images/arrow_down.gif\" alt=\"\" /></a></td>";
			}
			
			// Checkbox for selection
			if (strpos ($def["listview"], "select")!==false) {
				
			
			//array ($this->id."_sort_col_id"=>$col, $this->id."_nav_idx"=>1, $this->id."_is_nav"=>1
			
				if (!$rowData["is_deleted"])
					$htmlTable.="<td><input ".($rowData["is_selected"] ? "checked" : "")." type=\"checkbox\" onclick=\"WUI.ajaxRequest ('".WSystem::getRequestURI(NULL, false, array ($this->id."_sort_col_id"=>1, $this->id."_nav_idx"=>1))."','wrap_".$this->id."',null,false,'".$this->id."_cmd=select&".$this->id."_nav_idx=".($idx+1)."');\"></td>";
				else
					$htmlTable.="<td></td>";
			}
			
						
			$htmlTable.="</tr>\n";
			$idx++;
			$y++;
		}
		
		$cnt = round (count ($dataList) / $rowCnt);
		if (($cnt * $rowCnt)<count ($dataList))
			$cnt++;
	
		if ($page>0)
			$linkNavLast  = $this->createNavURI(($page-1)*$rowCnt+1);
		if ($page<($cnt-1))
			$linkNavNext  = $this->createNavURI(($page+1)*$rowCnt+1); 
		$linkNavEnd   = $this->createNavURI(count ($dataList));
		$linkNavStart = $this->createNavURI(1);		
		
		$i   = round (($navCnt-1) / 2);
		$i0  = $page-$i;
		$i1  = $page+$i+1;
		
		if ($i0<0) {
			$i0 = 0;
			$i1 = $navCnt;
			if ($i1>$cnt) 
				$i1=$cnt;
		}
		else
		if ($i1>$cnt) {
			$i1 = $cnt;
			$i0 = $cnt-$navCnt;	
			if ($i0 < 0) 
				$i0=0;
		}

		if (!empty ($linkNavLast))
			$htmlNavBar.="<a href=\"#\" onclick=\"WUI.ajaxRequest ('".$linkNavLast."','wrap_".$this->id."');return false;\">&lt;&lt;&nbsp;<!-->".WLanguage::BACK."<!--></a>&nbsp;&nbsp;<a href=\"#\" title=\"<!-->".WLanguage::FIRST_ENTRY."<!-->\" onclick=\"WUI.ajaxRequest ('".$linkNavStart."','wrap_".$this->id."');return false;\">...</a>&nbsp;"; 
		else
		if ($i1>1)
			$htmlNavBar.="&lt;&lt;&nbsp;".WLanguage::BACK."</a>&nbsp;";
	
		if ($i1>1) {
			$page0 = $i0;
			for ($i=0;$i<$navCnt;$i++) {
				if ($page0==$i1)
					break;
				if ($page0==$page)
					$htmlNavBar.="&nbsp;<b>".($page0+1)."</b>&nbsp;";
				else
					$htmlNavBar.="&nbsp;<a href=\"#\" onclick=\"WUI.ajaxRequest ('".$this->createNavURI($page0*$rowCnt+1)."','wrap_".$this->id."');return false;\">".($page0+1)."</a>&nbsp;";
				$page0++; // 
			} 
		}

		if (!empty ($linkNavNext)) 				
			$htmlNavBar.="&nbsp;<a href=\"#\" title=\"<!-->".WLanguage::LAST_ENTRY."<!-->\" onclick=\"WUI.ajaxRequest ('".$linkNavEnd."','wrap_".$this->id."');return false;\">...</a> &nbsp;<a href=\"#\" onclick=\"WUI.ajaxRequest ('".$linkNavNext."','wrap_".$this->id."');return false;\"><!-->".WLanguage::NEXT."<!-->&nbsp;&gt;&gt;</a>";
		else
		if ($i1>1)
			$htmlNavBar.="&nbsp;<!-->".WLanguage::NEXT."<!-->&nbsp;&gt;&gt;";


		$html = "";
		if ($htmlNavBar!="")
			$html.="<p align=\"center\">".$htmlNavBar."</p>";
		$html.="<table class=\"tblNavList\" style=\"width:".$def["width"]."\">".$htmlTable."</table>\r\n";

		if ($this->wrapListFunc) {
			$html = call_user_func ($this->wrapListFunc, $html, $this->id."_dataList"); 
		}
		
		if ((int) WSystem::getParam ($this->id."_is_ajax")) {
			$this->html = $html;
			WUI::updateArea ("wrap_".$this->id, $html);
		}
		
		$this->html = "<div id=\"wrap_".$this->id."\">".$html."</div>";
		
		unset ($_SESSION[$this->id."_highlightKeyValue"]);
	}
		
	public function getSize () {
		if (isset ($_SESSION[$this->id."_dataList"])) {
			return count ($_SESSION[$this->id."_dataList"]);
		}
		return 0;
	}
		
	public function getData ($idx) {		
		if (isset ($_SESSION[$this->id."_dataList"][$idx-1])) {
			return $_SESSION[$this->id."_dataList"][$idx-1];
		}
		return NULL;
	}
	
	public function getSelectedData () {
		return $this->getData((int) WSystem::getParam ($this->id."_nav_idx"));
	}
	
	public function setData ($idx, $data) {
		if ($this->rowDataFunc) {
			$tmp = call_user_func ($this->rowDataFunc, $data);
			if ($tmp!=NULL) {				
				$data = $tmp;
			}
		}

		if (isset ($_SESSION[$this->id."_dataList"][$idx-1])) {
			unset ($_SESSION[$this->id."_dataList"][$idx-1]["is_deleted"]);
			$_SESSION[$this->id."_dataList"][$idx-1]=array_merge ($_SESSION[$this->id."_dataList"][$idx-1], $data);
		}
	}

	public function appendData ($idx, $data, $fTop=false) {
		if ($this->rowDataFunc) {
			$tmp = call_user_func ($this->rowDataFunc, $data);
			if ($tmp!=NULL) {				
				$data = $tmp;
			}
		}

		if (isset ($_SESSION[$this->id."_dataList"][$idx-1])) {
		
			$max = count($_SESSION[$this->id."_dataList"]);
			for ($i=$max;$i>$idx;$i--) {
				$_SESSION[$this->id."_dataList"][$i]=$_SESSION[$this->id."_dataList"][$i-1];
			}
			$_SESSION[$this->id."_dataList"][$idx]=$data;
			
			if ($fTop) {
				$tmp = $_SESSION[$this->id."_dataList"][$idx-1];
				
				$_SESSION[$this->id."_dataList"][$idx-1]=$data;
				$_SESSION[$this->id."_dataList"][$idx]=$tmp;
			}

		}
	}
	
	
	public function setSelectedData ($data) {
		$this->setData ((int) WSystem::getParam ($this->id."_nav_idx"), $data);
	}
	
	public function getSelectedIdx () {
		return (int) WSystem::getParam ($this->id."_nav_idx");
	}
	
	public function reset ($searchData=array()) {

		$this->setSearchData ($searchData);		
		$def = $this->getDefinition ();		
		$_SESSION[$this->id."_dataList"]=$this->loadDataList ();
		unset ($_SESSION[$this->id."_sort_col_id"]);
		unset ($_SESSION[$this->id."_sort_direction"]);
		// Ingnore nav_idx in request uri
		WSystem::updateRequestURI (array (), false, array ($this->id."_nav_idx"=>1));
		unset ($_GET[$this->id."_nav_idx"]);
	}
	
	public function setSearchData ($searchData) {
		$_SESSION[$this->id."_searchdata"] = $searchData;
	}
	
	public function getSearchData () {
		if (isset ($_SESSION[$this->id."_searchdata"]))
			return $_SESSION[$this->id."_searchdata"];
		return array ();
	}

	public function __construct ($id, $resetMode=RESET_NO_VIEW) {
		parent::__construct ($id);
		$this->resetMode = $resetMode;
		if (WSystem::getParam ("view")=="init") {
			unset ($_SESSION[$this->id."_searchdata"]);
			unset ($_SESSION[$this->id."_dataList"]);
			unset ($_SESSION[$this->id."_sort_col_id"]);
			unset ($_SESSION[$this->id."_sort_direction"]);	
		}
	}
	
	public function setRowStyleFunc ($rowStyleFunc) {
		$this->rowStyleFunc = $rowStyleFunc;

	}

	public function setWrapListFunc ($wrapListFunc) {
		$this->wrapListFunc = $wrapListFunc;

	}


	public function setRowDataFunc ($func) {
		$this->rowDataFunc = $func;
	}

	
	public function setHighlightNew ($key, $value) {
		$_SESSION[$this->id."_highlightKeyValue"] = array ($key, $value);
	}
	
	
	public function __destruct () {
	
	}
};

?>