<?php


class SQLScriptForm extends WForm
{
	const FORM_NAME = "sql_script_form";
	
	private $sql = "";
	
	private static $INPUTFIELDS = array 
	(			

		"sql_script" => "text_big;m"
	);


	public function __construct ($sql) 
	{
		$this->sql = $sql;
		parent::__construct (self::FORM_NAME, "edit");
	}
	
	public function __destruct () 
	{
		
	}
	
	public function getDefinition () {
		return self::$INPUTFIELDS;
	}

	public function createMask ($cmd, $values) 
	{
	
		$btClose = WUI::createButton ("Abbrechen", "abort", WUI::BUTTON_DEFAULT, array ("extra-class"=>"delete_button", "onclick"=>"WUI.closePopup('popup_sql_script');"), true);

		return 

				WUI::createInputFrame (WUI::INPUT_FRAME_COLS, array ("id"=>$this->id."_input_frame1"),

					WUI::createTextArea ("Notwendige SQL-Befehle", "sql_script", $values["sql_script"], array ("style"=>"width:650px;height:470px;"))
		    ).
		    '<div style="text-align:center;">'.WUI::createButton ("SQL Ausführen", "edit", WUI::BUTTON_SUBMIT, array ("extra-class"=>"submit edit_button"), true)." ".$btClose.'</div>';
	}		

	public function getNewData () {
		return NULL;
	}
	
	public function getEditData () {
		return array ("sql_script" => $this->sql);
	}

	public function insert ($values) {
	
		return false;
	}
	
	public function update ($values) {
		global $wdb;
		
		$errCnt=0;
		$errText="";
		$successCnt=0;
		
		// Execute sql scripts
		$sqlList = explode (";", $values["sql_script"]);		
		foreach ($sqlList as $sql) {
		
			$sql=trim ($sql);
			if (empty ($sql))
				continue;
			try {
				
				$wdb->query ($sql);			
				$successCnt++;	
			}
			catch (Exception $e) {
				$errCnt++;
				$errText.=$e->getMessage ()."\n";
			}		
		}
		
		
		if ($errCnt) {
			// Show sql errors
			$this->errText = $errCnt." Fehler beim Ausführen der SQL-Befehle.";			
			
			$btClose = WUI::createButton (WLanguage::CLOSE, "abort", WUI::BUTTON_DEFAULT, array ("extra-class"=>"delete_button", "onclick"=>"WUI.closePopup('popup_sql_error');"), true);

			WUI::openPopup ("popup_sql_error", "SQL-Fehler", 
				'<div style="max-height:160px;overflow:auto;">'.nl2br($errText).'</div>
				<div style="position:absolute;bottom:10px;">'.$btClose.'</div>', 
				array ("align" => "center", "width" => 400, "height"=>200, "is_moveable" => 1, "is_no_modal"=>1));

			return false;
		}
		
		// Sql scripts succeed
		$this->successText = $successCnt." SQL-Befehle erfolgreich ausgeführt. Bitte Seite neu laden.";
		WUI::closePopupAsync ("popup_sql_script");
		return true;
	}
	
	public function search ($values) {		
		
		
		return false;
	}
		
	public function delete ($values) {
		return false;
	}
}



class UniversalDB
{
	private static $ignoreDrop = array ("has_children", "sort_pos");

	public function __construct (&$logic, $attr=array ()) {	
		global $wdb;
		
		$sql = "";
			
		$table = DB_TABLE_PREFIX.$logic->props["name"];		
		
		$tableList = $this->getTables ();
		// Check if table exists		
		$fAvailable =isset ($tableList[$table]);
		
		if (!$fAvailable) {
			$sql = UniversalDB::getTableCreateSQL ($logic);
			if (isset ($attr["insert_defaults"]))
				$sql.="\r\n".$attr["insert_defaults"]; 
			$errCnt++;
		}
		else {
			// Check columns
			$columnList = self::getTableColumns ($table);
			foreach ($columnList as $column => $columnData) {
				if (!isset ($logic->inputDef[$column]) && !in_array ($column, self::$ignoreDrop)) {
					// Old column, drop it
					$sql.= "alter table ".$table." drop ".$column.";\n";
					$errCnt++;
				}
			}
			foreach ($logic->inputDef as $field => $format) {
				$list = explode (";", $format);
				$mode = $list[1];
				if ($mode=="&")
					continue;
				if (!isset ($columnList[$field])) {
					// Append column
					$sql.= "alter table ".$table." add ".self::getColumnSQL ($field, $format).";\n";
					if ($field=="created") {
						$sql.= "update ".$table." set created=NOW();\n";
					}
					$errCnt++;
				}
			}		
			
			if ($logic->fTreeView) {
				// Mandatory columns for treeview
				if (!isset ($columnList["has_children"])) {
					$sql.= "alter table ".$table." add ".self::getColumnSQL ("has_children", "b;*;0").";\n";			
					$errCnt++;		
				}
			}						
		}
		
		// Relation tables
		if (isset ($logic->props["relations"])) {
			foreach ($logic->props["relations"] as $table2 => $tableProps) {

				if ($tableProps["relation"]=="n:m") {
			
					$table3 = $logic->props["name"]."2".$table2;
					
					if (!isset ($tableList[DB_TABLE_PREFIX.$table3])) {
						// Create relation table
						$sqlTable = 	
							// "DROP TABLE IF EXISTS `".DB_TABLE_PREFIX.$logic->props["name"]."`;\n".
							"CREATE TABLE `".DB_TABLE_PREFIX.$table3."` (\n".
							"   `".$logic->props["primary_column"]."` INT unsigned NOT NULL,\n".
							"   `".$tableProps["primary_column"]."` INT unsigned NOT NULL,\n".
							"   `sort_pos` INT default 0,\n".
							"   PRIMARY KEY  (`".$logic->props["primary_column"]."`,`".$tableProps["primary_column"]."`)\n".
							") ENGINE=InnoDB DEFAULT CHARSET=utf8;\n";


						if (!empty ($sql)) {
							$sql.="\n";
						}
						$sql.=$sqlTable;
						$errCnt++;
					}
				}
			}
		}	
		if ($errCnt) {
			$form = new SQLScriptForm ($sql);
			WUI::openPopup ("popup_sql_script", "SQL-Script ausführen", $form->__toString (), array ("align" => "center", "width" => 700, "height"=>600, "is_moveable" => 1, "is_no_close" => 1));
		}
	}	
	
	public static function getTableColumns ($table) {
		global $wdb;
		
		$columnList = array ();
		$sql = "show columns from ".$table;
		if ($wdb->query ($sql)) {
			$data = array ();
			while ($wdb->fetch ($data)) {
				$columnList[$data["Field"]]=$data;
			}
		}		
		return $columnList;	
	}
	
	public static function getTables () {
		global $wdb;
		$sql = "show tables;";
		
		$list = array ();
		if ($wdb->query ($sql)) {
			$data = array ();
			while ($wdb->fetch ($data)) {
				$table=$data["Tables_in_".WSystem::getConf("db_name")];
				$list[$table]=$data;
			}
		}		
		return $list;	
	}

	public static function getColumnSQL ($field, $format)
	{
		$sqlColumn = "";
		$list = explode (";", $format);
			
		$type = $list[0];
		$fMandarory = $list[1]=="m";
		
		$default = WFormat::getEmptyValue ($type, true);
		if (isset ($list[2]) && !empty ($list[2]))
			$default = $list[2];
		
		$sqlType = WFormat::getSQLType ($type);
		$sqlColumn = "`".$field."` ".$sqlType;
		
		if ($field=="created")
			$sqlColumn.=" default CURRENT_TIMESTAMP";				
		else
		if ($fMandarory)
			$sqlColumn.=" NOT NULL";
		else
		if ($sqlType!="TEXT") {
			$sqlColumn.=" default '".$default."'";
		}
		return $sqlColumn;
	}
	
	public static function getTableCreateSQL (&$logic) {
		
		// Check existing db table
		$sql = "";
		foreach ($logic->inputDef as $field => $format) {
			if ($field==$logic->props["primary_column"])
				continue;
			$list = explode (";", $format);
			$mode = $list[1];
			if ($mode=="&")
				continue;

			$sqlColumn = self::getColumnSQL ($field, $format);

			$sql.="   ".$sqlColumn.",\n";
		}
		
			
		if ($logic->fTreeView) {
			// Mandatory columns for treeview
			$sql.="   ".self::getColumnSQL ("has_children", "b;*;0").",\n";
		}

		// Primary key
		$sql.="   `".$logic->props["primary_column"]."` INT unsigned NOT NULL auto_increment,\n   PRIMARY KEY  (`".$logic->props["primary_column"]."`)";

		// Create table
		$sqlTable = 	
			// "DROP TABLE IF EXISTS `".DB_TABLE_PREFIX.$logic->props["name"]."`;\n".
			"CREATE TABLE `".DB_TABLE_PREFIX.$logic->props["name"]."` (\n".$sql.
			") ENGINE=MyISAM AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;\n";

		return $sqlTable;
	}
	
}


?>